/** @module recline */


exports.includes = function(){
	this.url = require("url");
	this.http = require("http");
	this.couchdb = require('./vendor/couchdb');
	this._ = require("underscore");
}

/**
	Sets the project base path for recline
 */
exports.setBasePath = function(dirname) {
	this.dirname = exports.parsePathForTrailingSlash(dirname);
}

/**
	Removes trailing Slashes
	@type utility
 */
exports.parsePathForTrailingSlash = function(path) {
	var $ = path;
	if($.lastIndexOf("/")+1 != $.length){
		$ = $ + "/";
	}
	return $;
}

/**
 	Sets a temporary directory for file uploads
 */
exports.setTempDir = function(relativeDirPathToReclineBase) {
	var $ = exports.parsePathForTrailingSlash(relativeDirPathToReclineBase);
	this.tempDir = this.dirname ? this.dirname + $ : __dirname + $;
}

/**
    <strong>Recline.setup</strong>
    <p>The main entry point for a recline object. It takes a couchDB
    HTTP string as the connection OR an evironment variable.
    </p>
    <ul>
        <li><strong>String: </strong> eg. "http://user:pass@127.0.0.1:5984"</li>
    </ul>
		@param url connection string
 */
exports.setup = function(url) {
	this.includes();
	if(typeof this.tempDir === "undefined") {
		this.setTempDir("/tmp/");
	}
	
	this.databases = {};
	if(typeof this.uri === "object"){
		throw new Error("cannot replace connection");
	}
	
	if(typeof url === "string") {			
		var uri = this.url.parse(url);
		if(!uri.protocol) {
			throw new Error("must be http protocol");
		}else
		{
			this.uri = uri;
		}
		this._.extend(this.uri, this._auth(this.uri.auth));
		return this.uri;
		
	}else{
		throw new Error("setup requires connection string");
	}
	return true;
}

/**
	Sets a database prefix. 
	Always ends with a dash "-"
	@type utility
	@default none
*/
exports.set_prefix = function(pf){
	this._.extend(this.uri, { "prefix": pf+"-" });
}

/**
	Returns the url prefix
	@return prefix
*/
exports.get_prefix = function(){
	return this.uri.prefix;
}

/**
	Gets all dbs in couchdb:
	Really just a wrapper function
	@param fn callback
*/
exports.dbs = function(fn){
	
	var o = this._.clone(this.uri);
	this._.extend(o, { path: "/_all_dbs", 
										 host: this.uri.hostname, 
										 headers: { "Authorization" : this.uri.basic } });
										
	this.http.get(o, function(res) {
		var callback = fn;
		res.setEncoding('utf8');
	  res.on('data', function(chunk){
			fn.call(this, chunk);
		});
	});
	
}

/**
	<strong>Creates a new DB and adds it to recline</strong>
	<p>It stores the database references as recline[name] &
	also in a databases cache</p>
	@param name database name
	@param fn callback 
*/
exports.createdb = function(name, fn){
	// I need to makesure that this is cached everytime that a database is created
	// it should store it for later use - possibly cradle cache funtion?
	var o = this._.clone(this.uri);
	var self = this;
	if(typeof this[name] === "object"){
		if(typeof this[name].db == "object"){
			return this[name];
		}
	}
	
	var auth = o.auth.split(":");
	auth = { user : auth[0], pass: auth[1] };
	this._client = new this.couchdb.createClient(o.port, o.hostname, auth.user, auth.pass);
	this[name] = self._client.db(this.uri.prefix + name);
	this.databases[name] = this[name];
	var db = this[name]
	this[name].exists(function(err,ok){
		if(ok){
			if(typeof fn === "function"){
				fn.call(this, null,db);
			}
		}else{
			self._client.request('PUT', '/'+self.uri.prefix + name);
			fn.call(this, null,db);
		}
	})

}

/**
	Destorys all the databses created by recline
	<strong>its permanent, I use it for testing</strong>
	@param {function} fn callback
*/
exports.destroyAll = function(fn) {
	for(var key in this.databases){
		this.databases[key].remove();
	}
}

/**
	Utility function for generate base 64 auth strong
	@type util
	@return base64 auth
*/
exports._auth = function(auth) {
  return { "basic" : "Basic " + new Buffer(auth).toString('base64') }
}


